#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Simple 2D wave-propagation test in 2 materials

import waves as w
import tbox
import tbox.gmsh as gmsh
import waves.srcs as srcs
import waves.utils as u


def model(p={}):
    # configure/build mesh
    pars = {
        'Lx': p['Lx'],
        'Ly1': p['Ly1'],
        'Ly2': p['Ly2'],
        'Nx': p['Nx'],
        'Ny1': p['Ny1'],
        'Ny2': p['Ny2']
    }
    msh = gmsh.MeshLoader("cube2mat.geo", __file__).execute(**pars)
    gmshWriter = tbox.GmshExport(msh)
    pbl = w.Problem(msh)

    # compute time-step size
    Lex = float(p['Lx']) / p['Nx']
    Ley1 = float(p['Ly1']) / p['Ny1']
    Ley2 = float(p['Ly2']) / p['Ny2']
    dt1 = min(Lex, Ley1) / p['c1']
    dt2 = min(Lex, Ley2) / p['c2']
    dt = min(dt1, dt2) * 0.8
    #lam = maxc / p['freq']

    pbl.add(w.Medium(msh, "Volume1", p['c1']))
    pbl.add(w.Medium(msh, "Volume2", p['c2']))

    if p['bc'] == 'sinsrc':  # source de bas en haut
        pbl.add(srcs.SinSource(msh, "FaceY0", p['freq']))
        pbl.add(w.Boundary(msh, "FaceY1", p['c2']))
        pbl.add(w.Boundary(msh, "FaceY0", p['c1']))

    elif p['bc'] == 'interf1':  # interference de 2 sources sur la frontière basse
        s = srcs.MultiSource(msh, "FaceY0")
        s.addSource(tbox.Vector3d(3, 0, 0), p['freq'] / 2)
        s.addSource(tbox.Vector3d(7, 0, 0), 2 * p['freq'])
        pbl.add(s)
        pbl.add(w.Boundary(msh, "FaceX0_1", p['c1']))
        pbl.add(w.Boundary(msh, "FaceX0_2", p['c2']))
        pbl.add(w.Boundary(msh, "FaceX1_1", p['c1']))
        pbl.add(w.Boundary(msh, "FaceX1_2", p['c2']))
        pbl.add(w.Boundary(msh, "FaceY1", p['c2']))

    elif p['bc'] == 'pulse':  # un pulse qui part du coin inf gauche
        pbl.add(
            srcs.PulseSource(msh, "FaceY0", p['freq'], shift=0.3, width=200.0))
        #pbl.add(w.Boundary(msh, "FaceX0_1", p['c1']))
        #pbl.add(w.Boundary(msh, "FaceX0_2", p['c2']))
        #pbl.add(w.Boundary(msh, "FaceX1_1", p['c1']))
        #pbl.add(w.Boundary(msh, "FaceX1_2", p['c2']))
        pbl.add(w.Boundary(msh, "FaceY1", p['c2']))
        pbl.add(w.Boundary(msh, "FaceY0", p['c1']))

    elif p['bc'] == 'reflex':  # réflexion/réfraction (Bulthuis)
        pbl.add(srcs.SinSource(msh, "Line1", p['freq']))
        #pbl.add(w.Boundary(msh, "FaceX0_1", p['c1']))
        #pbl.add(w.Boundary(msh, "FaceX0_2", p['c2']))
        pbl.add(w.Boundary(msh, "FaceX1_1", p['c1']))
        pbl.add(w.Boundary(msh, "FaceX1_2", p['c2']))
        pbl.add(w.Boundary(msh, "FaceY1", p['c2']))
        #pbl.add(w.Boundary(msh, "FaceY0", p['c1']))
    else:
        raise Exception('Unknown BC type (%s)' % p['bc'])

    scheme = w.ForwardEuler(pbl)
    #scheme.wgmsh = w.Wgmsh_WG_ALL # write .pos files
    scheme.ttot = p['ttot']
    scheme.dt = dt
    scheme.savefreq = 5

    return scheme, gmshWriter


def runmodel(**d):
    p = {}
    p['bc'] = 'reflex'
    p['ttot'] = 1.0
    p['Lx'] = 10.0
    p['Ly1'] = 5.0
    p['Ly2'] = 5.0
    p['Nx'] = 100
    p['Ny1'] = 50
    p['Ny2'] = 50
    p['c1'] = 10.0
    p['c2'] = 20.0
    p['freq'] = 10.0
    p.update(d)

    scheme, writer = model(p)
    u.integrate(scheme, writer)


if __name__ == "__main__":
    runmodel()
